/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.layout.page.template.admin.web.internal.util.comparator;

import com.liferay.asset.display.page.model.AssetDisplayPageEntry;
import com.liferay.portal.kernel.util.DateUtil;
import com.liferay.portal.kernel.util.OrderByComparator;

/**
 * @author Rubén Pulido
 */
public class AssetDisplayPageEntryModifiedDateComparator
	extends OrderByComparator<AssetDisplayPageEntry> {

	public static final String ORDER_BY_ASC =
		"AssetDisplayPageEntry.modifiedDate ASC";

	public static final String ORDER_BY_DESC =
		"AssetDisplayPageEntry.modifiedDate DESC";

	public static final String[] ORDER_BY_FIELDS = {"modifiedDate"};

	public AssetDisplayPageEntryModifiedDateComparator() {
		this(true);
	}

	public AssetDisplayPageEntryModifiedDateComparator(boolean ascending) {
		_ascending = ascending;
	}

	@Override
	public int compare(
		AssetDisplayPageEntry assetDisplayPageEntry1,
		AssetDisplayPageEntry assetDisplayPageEntry2) {

		int value = DateUtil.compareTo(
			assetDisplayPageEntry1.getModifiedDate(),
			assetDisplayPageEntry2.getModifiedDate());

		if (_ascending) {
			return value;
		}

		return -value;
	}

	@Override
	public String getOrderBy() {
		if (_ascending) {
			return ORDER_BY_ASC;
		}

		return ORDER_BY_DESC;
	}

	@Override
	public String[] getOrderByFields() {
		return ORDER_BY_FIELDS;
	}

	@Override
	public boolean isAscending() {
		return _ascending;
	}

	private final boolean _ascending;

}